// The customer manager is hosted on firebase
// thats why we need these types and tools
//
import { firebase } from '@firebase/app';
import '@firebase/auth';
import '@firebase/firestore';
import { get } from 'lodash';
/**
 * Use EvaLogin widget to ask user to select an OAuth provider
 * login in and then we receive the firebase authentication session
 *
 * @param {springtree_solutionsEvaSuiteUiEvaLogin} widget The login widget instance to use for user interaction
 */
const authenticateWithOAuthProviders = (options) => {
    const authentication = firebase.auth();
    const loginWithGoogle = {
        id: 'google',
        label: 'Login with Google',
        data: {},
    };
    const loginWithMicrosoft = {
        id: 'microsoft',
        label: 'Login with Microsoft',
        data: {},
    };
    const loginWithGithub = {
        id: 'github',
        label: 'Login with Github',
        data: {},
    };
    options.loginWidget.showChoice({
        question: 'How do you want to authenticate?',
        label: 'Choose authentication provider',
        answers: [loginWithGoogle, loginWithMicrosoft, loginWithGithub],
    });
    // Listen for the users' choice
    //
    options.loginWidget.once('answer', async (answer) => {
        options.loginWidget.close();
        switch (answer.id) {
            case loginWithGoogle.id:
                await authentication.signInWithPopup(new firebase.auth.GoogleAuthProvider());
                break;
            case loginWithMicrosoft.id:
                await authentication.signInWithPopup(new firebase.auth.OAuthProvider('microsoft.com'));
                break;
            case loginWithGithub.id:
                await authentication.signInWithPopup(new firebase.auth.GithubAuthProvider());
                break;
            default:
                alert('Unknown authentication provider');
                break;
        }
    });
};
/**
 * Tries to retrieve an endpoint from the EVA customer manager
 * This will present a number of choices to the user and requires an OAuth login
 *
 * @returns {(Promise<string|undefined>)}
 */
export const getEndpointFromEvaCustomerManager = (options) => {
    return new Promise((resolve, reject) => {
        firebase.initializeApp(options.firebaseConfig);
        const authentication = firebase.auth();
        if (!authentication) {
            throw new Error('Missing authorisation support');
        }
        // Listen to Firebase authentication changes
        //
        const firebaseAuthenticationListener = authentication.onAuthStateChanged(async (user) => {
            if (user) {
                // Stop listening to Firebase authentication changes
                //
                firebaseAuthenticationListener();
                const db = firebase.firestore();
                // Select all available customers from FireStore
                //
                const customers = await db.collection('customer').get();
                // Ask user to choose a customer
                //
                options.loginWidget.showChoice({
                    question: 'Select EVA customer',
                    label: 'EVA Customer',
                    answers: customers.docs.map((item) => {
                        const data = item.data();
                        return {
                            data,
                            id: data.name,
                            label: data.name,
                        };
                    }),
                });
                // Listen for the answer of the customer selector
                //
                options.loginWidget.once('answer', async (answer) => {
                    const name = answer.label;
                    const environments = get(answer, 'data.baseEnvironments', []);
                    // If there is only one environment
                    // Select the only environment available
                    //
                    if (environments.length === 1) {
                        options.loginWidget.close();
                        resolve(environments[0].endpoint);
                    }
                    else if (environments.length > 2) {
                        // Retrieve the full details of the base environments
                        //
                        const baseEnvPromises = environments.map((baseEnvRef) => {
                            return baseEnvRef.get().then(snapshot => snapshot.data());
                        });
                        const baseEnvironments = await Promise.all(baseEnvPromises);
                        // Ask to select an environment
                        //
                        options.loginWidget.showChoice({
                            question: `Select ${name} environment`,
                            label: 'Environment',
                            answers: baseEnvironments.map((data) => {
                                return {
                                    id: data.type,
                                    label: data.type,
                                    data: data.endpoint,
                                };
                            }),
                        });
                        // Bootstrap the EVA SDK with the selected endpoint
                        //
                        options.loginWidget.once('answer', async (answer) => {
                            options.loginWidget.close();
                            resolve(answer.data);
                        });
                    }
                });
            }
            else {
                authenticateWithOAuthProviders(options);
            }
        });
    });
};
//# sourceMappingURL=customer-manager.js.map