import { find } from 'lodash';
import * as uuid from 'uuid';
import { createStoreObservable } from '@springtree/eva-sdk-redux-store-builder';
import { createObservableForPath } from '../utils/pathObservable';
import { bootstrapEndpoint } from '@springtree/eva-sdk-core-service';
import { PreferenceManager } from '../utils/preference';
/**
 * The session manager is used to create and manage session for the React Context.
 *
 * @export
 * @class EvaSessionManager
 */
export class EvaSessionManager {
    constructor() {
        /**
         * The collection of EVA sessions that can be swapped into and out of the React context
         *
         * @private
         * @type {{[key:string]: IEvaSession =}
         */
        this.sessions = [];
    }
    /**
     * Creates a new EVA Context session by creating the Redux store and bootstrapping it
     * Caller can supply a session identifier or one will be created automatically
     *
     * @param {IEvaSessionOptions} options
     * @returns {Promise<IEvaSession>}
     */
    async createSession(builder, options) {
        // Use provided session ID or generate a new one
        //
        const sessionId = options.sessionId || uuid.v4();
        // Create the Redux store and its observable
        //
        const store = await builder.createStore();
        const store$ = createStoreObservable(store);
        // Create the preference manager and observable
        //
        const preferences = new PreferenceManager(store, options.storage);
        const session = {
            sessionId,
            preferences,
            preferences$: preferences.preferences$,
            store: store,
            store$: store$,
            storage: options.storage,
            currentUserData$: createObservableForPath(store$, 'currentUser.data.response'),
            currentUserStatus$: createObservableForPath(store$, 'currentUserStatus'),
            currentUserToken$: createObservableForPath(store$, 'currentUserToken'),
            getEvaEndpoint: async () => {
                const endPointUrl = store.getState().endpointUrl;
                const endPoint = await bootstrapEndpoint({ uri: endPointUrl });
                return endPoint;
            },
        };
        this.sessions.push(session);
        // Activate the session if requested which is the default
        //
        if (!options.createOnly) {
            this.currentSession = session;
            // Update session ID in local storage
            //
            if (options.storage) {
                options.storage.setItem('evaSuite:sessionId', session.sessionId);
            }
        }
        return session;
    }
    /**
     * Retrieves a session from the pool by it's session identifier
     *
     * @param {string} sessionId
     * @returns {(IEvaSession|undefined)}
     */
    getSession(sessionId) {
        return find(this.sessions, { sessionId });
    }
}
//# sourceMappingURL=manager.js.map