import { EvaEndpoint } from '@springtree/eva-sdk-core-service';
import { EvaStorage } from '@springtree/eva-sdk-core-storage';
import { IEvaCurrentUserStatus, IEvaReduxStoreDefaultShape } from '@springtree/eva-sdk-redux-store-builder';
import { Store } from 'redux';
import { BehaviorSubject, Observable } from 'rxjs';
import { PreferenceManager, IUserPreferences } from '../utils/preference';
import { IEvaSuiteStorage } from '../utils/storage';
/**
 * The current React context will be populated with a single EVA session.
 * A session is:
 * - an EVA endpoint (url)
 * - a redux store
 * - current user details (from the store)
 *
 * All the data mentioned above is available and managed in the Redux store
 * Convenience methods will be provided for quick access to these details with observables
 *
 * @export
 * @interface IEvaSession
 */
export interface IEvaSession<STORE extends IEvaReduxStoreDefaultShape, STORAGE extends IEvaSuiteStorage> {
    /**
     * The session identifier from the `EvaSessionManager`
     * Can also be used in service call payload as a `SessionID` if said service supports that
     *
     * @type {string}
     */
    sessionId: string;
    /**
     * The Redux store for the EVA session
     *
     * @type {Store<IEvaAdminStore>}
     */
    store: Store<STORE>;
    /**
     * An observable version of the Redux store for the EVA session
     *
     * @type {BehaviorSubject<STORE>}
     */
    store$: BehaviorSubject<STORE>;
    /**
     * The preference manager for the session
     *
     * @type {PreferenceManager<STORE, STORAGE>}
     */
    preferences: PreferenceManager<STORE, STORAGE>;
    /**
     * The observable for the current user preferences
     *
     * @type {BehaviorSubject<IUserPreferences>}
     */
    preferences$: BehaviorSubject<IUserPreferences>;
    /**
     * Expose storage on the session as a convenience
     *
     * @type {EvaStorage}
     */
    storage?: EvaStorage<STORAGE>;
    /**
     * Set to an Authentication token if a user is logged in
     * Changes will trigger local storage updates
     *
     * @type {Observable<string>}
     */
    currentUserToken$: Observable<string>;
    /**
     * The current user details which can be a logged in user or the default anonymous user
     * See `currentStatus$` for helpers to quickly determine the type of logged in user
     *
     * @type {Observable<EVA.Core.GetCurrentUserResponse>}
     */
    currentUserData$: Observable<EVA.Core.GetCurrentUserResponse>;
    /**
     * Indicates the type of user currently logged in.
     * Provides convenience properties such as `isCustomer` and `isAnonymous`
     * See the typings for the full details
     *
     * @type {Observable<IEvaCurrentUserStatus>}
     */
    currentUserStatus$: Observable<IEvaCurrentUserStatus>;
    /**
     * Returns the current instance of EvaEndpoint based on the url in the store
     * Will bootstrap the EndPoint implicitly.
     * Calling services outside the Redux store will be the most obvious use for
     * the EvaEndpoint
     *
     */
    getEvaEndpoint: () => Promise<EvaEndpoint>;
}
//# sourceMappingURL=session.d.ts.map