import { evaResponseError } from '@springtree/eva-sdk-core-response-error';
import { bootstrapEndpoint } from '@springtree/eva-sdk-core-service';
import { setCoreSetting } from '@springtree/eva-sdk-core-settings';
import { evaWarningLogger } from '@springtree/eva-sdk-core-warning';
import { StoreBuilder, createEndpointAction, createCurrentUserUpdateAction, createServiceRequestLogic, createServiceReducer, } from '@springtree/eva-sdk-redux-store-builder';
import { isArray } from 'lodash';
import { getEndpointFromEvaCustomerManager, getEndpointFromRootConfig } from '../endpoint/index';
import { STORAGE_PREFIX } from '../utils/storage';
// Activate all the default service interceptors for Axios
//
// evaAvailableServices.activate();
evaResponseError.activate();
evaWarningLogger.activate();
export class EvaContextBuilder {
    /**
     * Creates an instance of EvaContextBuilder
     * Provide the template classes for your storage and store state interfaces
     *
     * @param {IEvaContextBuilderOptions<STORAGE>} options The options needed to create the store
     */
    constructor(options) {
        this.options = options;
        /**
         * The collection of redux logics to build the store with
         *
         * @private
         * @type {Logic<STORE>[]}
         */
        this.logic = [];
        /**
         * The collection of reducers to build the store with
         *
         * @private
         * @type {ReducersMapObject<Partial<STORE>>}
         */
        this.reducer = {};
        // Set our application details
        // Calling application MUST provide these
        //
        setCoreSetting('appName', options.appName);
        setCoreSetting('appVersion', options.appVersion);
    }
    /**
     * Purge all previously set reducers and logics
     *
     */
    resetReducerAndLogic() {
        this.logic = [];
        this.reducer = {};
    }
    /**
     * Helper method to add both the reducer and logic for an EVA service in one call
     *
     * @template SVC
     * @param {keyof STORE} path
     * @param {new () => SVC} service
     */
    addEvaService(path, service) {
        // Check we are not overwriting a previous reducer
        //
        if (this.reducer[path]) {
            throw new Error(`Duplicate reducer provided for ${path}`);
        }
        this.reducer[path] = createServiceReducer(service, { stateKey: path });
        this.logic.push(...createServiceRequestLogic(service, { stateKey: path }));
    }
    /**
     * Add a custom state reducer to the store
     *
     * @param {keyof STORE} path
     * @param {Reducer<STORE[keyof STORE]>} reducer
     */
    addReducer(path, reducer) {
        this.reducer[path] = reducer;
    }
    /**
     * Add custom store logic
     *
     * @param {Logic<STORE>} logic The redux logic that is to be added to the store
     */
    addLogic(logic) {
        if (isArray(logic)) {
            this.logic.push(...logic);
        }
        else {
            this.logic.push(logic);
        }
    }
    /**
     * Create a Redux store using the provided and previously configured reducers and logics
     *
     */
    async createStore() {
        console.info('[EVA-CONTEXT]: Bootstrapping EVA Store');
        const logicCount = this.logic.length;
        const reducerCount = Object.keys(this.reducer).length;
        console.info(`[EVA-CONTEXT]: Store with ${reducerCount} reducer(s) and ${logicCount} logic(s)`);
        // Create the default redux store with current user support
        // Also create the RxJS observable for it
        // Provide our storage instance to setup inter-app sync of user token en endpoint
        // If the endpoint and user token are present in local storage they will be populated in the store
        //
        const builder = new StoreBuilder();
        const store = builder.createStore({
            reducers: this.reducer,
            logics: this.logic,
            storage: this.options.storage,
            storageKeyPrefix: STORAGE_PREFIX,
        });
        // Figure out what the the endpoint needs to be
        // The procedure to do so is as follows:
        //
        // * check if an endpoint is set in the store
        // * check for an eva-config in the site root and use that
        // * use the EVA customer manager to select an EVA backend
        //
        console.info('[EVA-CONTEXT]: Checking EVA endpoint...');
        if (!store.getState().endpointUrl) {
            // No endpoint has been set for the current store
            // Time to fetch and check the EVA config
            //
            const rootConfigEndpoint = await getEndpointFromRootConfig();
            if (rootConfigEndpoint) {
                console.info('[EVA-CONTEXT]: Configure endpoint from site config', rootConfigEndpoint);
                const rootEndpointAction = createEndpointAction({ endpoint: rootConfigEndpoint });
                store.dispatch(rootEndpointAction);
            }
            else {
                // No EVA config could be found in the site root
                // We will get an environment selection from the EVA customer manager
                // The customer manager is a developer tool which requires a valid login
                // with an OAuth account
                //
                const customerManagerEndpoint = await getEndpointFromEvaCustomerManager(this.options);
                if (customerManagerEndpoint) {
                    console.info('[EVA-CONTEXT]: Configure endpoint from customer manager', customerManagerEndpoint);
                    const customerManagerEndpointAction = createEndpointAction({ endpoint: customerManagerEndpoint });
                    store.dispatch(customerManagerEndpointAction);
                }
                else {
                    throw new Error('Failed to initialise the application. Missing endpoint');
                }
            }
        }
        // We will have an endpoint url if we get here so let's bootstrap it
        //
        const endpointUrl = store.getState().endpointUrl;
        console.info('[EVA-CONTEXT]: Bootstrapping endpoint...', endpointUrl);
        await bootstrapEndpoint({ uri: endpointUrl, storage: this.options.storage });
        // Fetch anonymous or actual user depending on what was pulled out of storage
        //
        const updateUserAction = createCurrentUserUpdateAction();
        store.dispatch(updateUserAction);
        try {
            await updateUserAction.payload.promise;
        }
        catch (error) {
            console.error('[EVA-CONTEXT]: Error while fetching current user', error);
        }
        // We're done here, the store is usable
        //
        return store;
    }
}
//# sourceMappingURL=bootstrap.js.map