import { EvaStorage } from '@springtree/eva-sdk-core-storage';
import { IEvaReduxStoreDefaultShape } from '@springtree/eva-sdk-redux-store-builder';
import { Store } from 'redux';
import { BehaviorSubject } from 'rxjs';
import { IEvaSuiteStorage } from './storage';
/**
 * The recent items are a sliding array of 10 (configurable) unique ids
 *
 * @export
 * @interface IRecentItems
 */
export interface IRecentItems {
    DiscountID: number[];
    InvoiceID: number[];
    OrderID: number[];
    OrganizationUnitID: number[];
    ProductID: number[];
    SearchQuery: string[];
    UserID: number[];
}
/**
 * Every application has the option to store generic key/value settings
 * If an interface exists for the settings it will be declared and managed
 * within the application itself
 *
 * @export
 * @interface IAppSettings
 */
export interface IAppSettings {
    [appName: string]: {
        [key: string]: any;
    };
}
/**
 * For a single user we collect their recently viewed item IDs
 * search queries and any personal settings
 *
 * @export
 * @interface IUserPreferences
 */
export interface IUserPreferences {
    history: IRecentItems;
    settings: IAppSettings;
}
/**
 * The preferences are collected per username and stored in a singular storage
 * key with the below interfaces
 *
 * @export
 * @interface IPreferences
 */
export interface IPreferences {
    [username: string]: IUserPreferences;
}
export declare class PreferenceManager<STORE extends IEvaReduxStoreDefaultShape, STORAGE extends IEvaSuiteStorage> {
    private store;
    private storage;
    private storageKey;
    private historyLimit;
    /**
     * The current in-memory copy of the preferences
     *
     * @private
     * @type {IPreferences}
     */
    private _preferences;
    /**
     * Setter to capture changes and emit them on the observable
     *
     * @private
     */
    private set preferences(value);
    /**
     * Helper method to create empty initial preferences for Anonymous
     *
     * @private
     * @returns {IPreferences}
     */
    private createDefaultPreferences;
    /**
     * Internal getter to access the preferences and return a default empty object
     * if not initialized
     *
     * @private
     */
    private get preferences();
    /**
     * This is the RxJS observable you can subscribe to for the current users preferences
     * The whole preferences structure has keys per user
     * The observable will only return the structure for the current user
     *
     * @type {BehaviorSubject<IUserPreferences>}
     */
    preferences$: BehaviorSubject<IUserPreferences>;
    /**
     * Creates an instance of PreferenceManager
     *
     * @param {Store<STORE>} store The redux store which we need for the current user name
     * @param {EvaStorage<STORAGE>} storage The storage for the preferences
     */
    constructor(store: Store<STORE>, storage: EvaStorage<STORAGE>, storageKey?: keyof STORAGE, historyLimit?: number, storageEventEmitter?: Window & typeof globalThis);
    /**
     * Helper method to determine the current username based on the redux store
     *
     * @private
     * @returns {string}
     */
    private getCurrentUserName;
    /**
     * Helper method to get the current preferences
     * Can optionally be initialized for the current user
     *
     * @private
     * @param {string} [currentUserName] The username to initialize for
     * @returns {IPreferences}
     */
    private getCurrentPreferences;
    /**
     * This will add a recent item ID to the history preferences for the current user
     *
     * @param {keyof IRecentItems} key
     * @param {number} id
     */
    addRecentItem(key: keyof IRecentItems, id: number): void;
    /**
     * This will retrieve recent item IDs for the current user
     *
     * @param {keyof IRecentItems} key
     */
    getRecentItems(key: keyof IRecentItems): any;
    /**
     * This will add an application setting
     *
     * @param {string} appName The name of the application the setting is for
     * @param {string} key The name of the setting
     * @param {*} value The value of the setting
     */
    addAppSetting(appName: string, key: string, value: any): void;
    /**
     * Returns the current settings for an application.
     * More exact typings can be cast by the calling application
     *
     * @param {string} appName
     * @returns {{[key: string]: any}}
     */
    getAppSettings(appName: string): {
        [key: string]: any;
    };
}
//# sourceMappingURL=preference.d.ts.map