import { concat, isEqual, uniq, slice } from 'lodash';
import { BehaviorSubject } from 'rxjs';
export class PreferenceManager {
    /**
     * Creates an instance of PreferenceManager
     *
     * @param {Store<STORE>} store The redux store which we need for the current user name
     * @param {EvaStorage<STORAGE>} storage The storage for the preferences
     */
    constructor(store, storage, storageKey = 'evaSuite:preferences', historyLimit = 10, storageEventEmitter = window) {
        this.store = store;
        this.storage = storage;
        this.storageKey = storageKey;
        this.historyLimit = historyLimit;
        /**
         * The current in-memory copy of the preferences
         *
         * @private
         * @type {IPreferences}
         */
        this._preferences = {};
        // Subscribe to storage events to update the preferences
        //
        try {
            storageEventEmitter.addEventListener('storage', (storageEvent) => {
                if (storageEvent.key && storageEvent.key === storageKey) {
                    console.log(`[EVA-CONTEXT] new preferences from storage: ${storageEvent.newValue}`);
                    // Update in-memory preferences from storage
                    //
                    try {
                        const newValue = JSON.parse(storageEvent.newValue);
                        this.preferences = newValue;
                    }
                    catch (error) {
                        console.warn('[EVA-CONTEXT] failed to parse new preferences', error);
                    }
                }
            });
        }
        catch (storageEventListenerError) {
            console.warn('[EVA-CONTEXT] Failed to register storage event listener', storageEventListenerError);
        }
        // Get the current username from the store
        //
        let currentUsername = this.getCurrentUserName();
        // Create the behavior subject for the preferences and populate it for the current user
        //
        this.preferences$ = new BehaviorSubject(this.preferences[currentUsername]);
        // Subscribe to getCurrentUser changes to switch the preferences if a new
        // user logs in or out
        //
        store.subscribe(() => {
            const newUserName = this.getCurrentUserName();
            if (newUserName !== currentUsername) {
                // Change to new username and retrieve or initialize the new preferences
                //
                currentUsername = newUserName;
                const preferences = this.getCurrentPreferences(currentUsername);
                // Use the internal setter to update the whole object because this will
                // both update storage and emit on the observable
                //
                this.preferences = preferences;
            }
        });
    }
    /**
     * Setter to capture changes and emit them on the observable
     *
     * @private
     */
    set preferences(preferences) {
        if (!isEqual(preferences, this.preferences)) {
            this._preferences = preferences;
            // Update preferences in storage
            //
            this.storage.setItem(this.storageKey, JSON.stringify(this._preferences));
            // Emit the new user preferences
            //
            const currentUsername = this.getCurrentUserName();
            this.preferences$.next(this._preferences[currentUsername]);
        }
    }
    /**
     * Helper method to create empty initial preferences for Anonymous
     *
     * @private
     * @returns {IPreferences}
     */
    createDefaultPreferences() {
        const defaultPreferences = {
            history: {
                DiscountID: [],
                InvoiceID: [],
                OrderID: [],
                OrganizationUnitID: [],
                ProductID: [],
                SearchQuery: [],
                UserID: [],
            },
            settings: {},
        };
        return defaultPreferences;
    }
    /**
     * Internal getter to access the preferences and return a default empty object
     * if not initialized
     *
     * @private
     */
    get preferences() {
        return this._preferences || { Anonymous: this.createDefaultPreferences() };
    }
    /**
     * Helper method to determine the current username based on the redux store
     *
     * @private
     * @returns {string}
     */
    getCurrentUserName() {
        var _a, _b;
        const storeState = this.store.getState();
        const currentUser = storeState.currentUser;
        const currentUsername = ((_b = (_a = currentUser) === null || _a === void 0 ? void 0 : _a.data.response) === null || _b === void 0 ? void 0 : _b.User.EmailAddress) || 'Anonymous';
        return currentUsername;
    }
    /**
     * Helper method to get the current preferences
     * Can optionally be initialized for the current user
     *
     * @private
     * @param {string} [currentUserName] The username to initialize for
     * @returns {IPreferences}
     */
    getCurrentPreferences(currentUsername) {
        // Fetch the preferences object and current username
        //
        const preferences = this.preferences;
        // Initialize user preferences if required
        //
        if (currentUsername && !preferences[currentUsername]) {
            preferences[currentUsername] = this.createDefaultPreferences();
        }
        return preferences;
    }
    /**
     * This will add a recent item ID to the history preferences for the current user
     *
     * @param {keyof IRecentItems} key
     * @param {number} id
     */
    addRecentItem(key, id) {
        const currentUsername = this.getCurrentUserName();
        const preferences = this.getCurrentPreferences(currentUsername);
        // Push the id in the recent history
        // We will use some lodash magic to keep the history at 10 unique items
        //
        const currentItems = preferences[currentUsername].history[key];
        const newItems = slice(uniq(concat(currentItems, [id])), 0, this.historyLimit);
        preferences[currentUsername].history[key] = newItems;
        // Use the internal setter to update the whole object because this will
        // both update storage and emit on the observable
        //
        this.preferences = preferences;
    }
    /**
     * This will retrieve recent item IDs for the current user
     *
     * @param {keyof IRecentItems} key
     */
    getRecentItems(key) {
        const currentUsername = this.getCurrentUserName();
        const preferences = this.getCurrentPreferences(currentUsername);
        // Return a copy not the original
        //
        const currentItems = preferences[currentUsername].history[key];
        return JSON.parse(JSON.stringify(currentItems));
    }
    /**
     * This will add an application setting
     *
     * @param {string} appName The name of the application the setting is for
     * @param {string} key The name of the setting
     * @param {*} value The value of the setting
     */
    addAppSetting(appName, key, value) {
        const currentUsername = this.getCurrentUserName();
        const preferences = this.getCurrentPreferences(currentUsername);
        // Safety check that the value can be stringified
        //
        try {
            JSON.stringify(value);
        }
        catch (error) {
            console.warn(`[EVA-CONTEXT] Invalid application setting supplied for ${key}`, error);
            throw new Error(`Invalid application setting supplied for ${key}`);
        }
        if (!preferences[currentUsername].settings[appName]) {
            preferences[currentUsername].settings[appName] = {};
        }
        // Set the application setting
        //
        preferences[currentUsername].settings[appName][key] = value;
        // Use the internal setter to update the whole object because this will
        // both update storage and emit on the observable
        //
        this.preferences = preferences;
    }
    /**
     * Returns the current settings for an application.
     * More exact typings can be cast by the calling application
     *
     * @param {string} appName
     * @returns {{[key: string]: any}}
     */
    getAppSettings(appName) {
        const currentUsername = this.getCurrentUserName();
        const preferences = this.getCurrentPreferences(currentUsername);
        return preferences[currentUsername].settings[appName] || {};
    }
}
//# sourceMappingURL=preference.js.map