import { get } from 'lodash';
import { createServiceRequestAction, createServiceResetAction, createServiceResponseAction, } from '@springtree/eva-sdk-redux-store-builder';
import { createObservableForPath } from './pathObservable';
/**
 * Helper method to create an EVA Service context based on the Redux store
 *
 * @export
 * @template SVC
 * @param {keyof IEvaAdminStore} path
 * @param {IEvaSession} session
 * @returns {IEvaServiceContext<SVC>}
 */
export function createServiceContext(params) {
    const context = {
        getState: () => get(params.session.store.getState(), params.path),
        getState$: () => createObservableForPath(params.session.store$, `${params.path})`),
        getRequest$: () => createObservableForPath(params.session.store$, `${params.path}.data.request`),
        getResponse$: () => createObservableForPath(params.session.store$, `${params.path}.data.response`),
        isFetching$: () => createObservableForPath(params.session.store$, `${params.path}.fetching`),
        getError$: () => createObservableForPath(params.session.store$, `${params.path}.error`),
        fetch: (payload = params.defaultPayload) => {
            // Create and dispatch the REQUEST action for the service on the current
            // store for the EVA session
            //
            const action = createServiceRequestAction({
                payload,
                service: params.service,
                stateOptions: { stateKey: params.path },
            });
            params.session.store.dispatch(action);
            return action;
        },
        response: (payload) => {
            // Create and dispatch the RESPONSE action for the service on the current
            // store for the EVA session
            // This might be useful for manually updating a response if it has been
            // supplied from another source.
            //
            const action = createServiceResponseAction({
                data: payload,
                service: params.service,
                stateOptions: { stateKey: params.path },
                serviceContext: 'MANUAL',
                chainContext: 'MANUAL',
            });
            params.session.store.dispatch(action);
            return action;
        },
        reset: (newState) => {
            // Create and dispatch the RESET action for the service on the current
            // store for the EVA session
            // You can optionally supply what you want the new state to be if not empty
            //
            const action = createServiceResetAction({
                state: newState,
                stateKey: params.path,
            });
            params.session.store.dispatch(action);
            return action;
        },
        call: async (payload, options) => {
            const endPoint = await params.session.getEvaEndpoint();
            return await endPoint.callService(params.service, payload, options);
        },
    };
    return context;
}
//# sourceMappingURL=serviceContext.js.map